package org.zeroturnaround.javarebel.integration.catalina;
import org.zeroturnaround.javarebel.ClassResourceSource;
import org.zeroturnaround.javarebel.IntegrationFactory;
import org.zeroturnaround.javarebel.Plugin;
import org.zeroturnaround.javarebel.ServletIntegration;
import org.zeroturnaround.javarebel.integration.generic.ServletContextImplClassBytecodeProcessor;

/**
 * JRebel plugin for Catalina.
 * <p>
 * TOMCAT 4.1, 5.5, 6.0 (Tested on 4.1.37, 5.5.27, 6.0.14)
 * GLASSFISH 2.1, 3.0 (Tested on 2.1-b60e, 3-prelude, 3-preview)
 * JBOSS 4.2, 5.0 (Tested on 4.2.2 GA, 5.0.0 GA)
 * 
 * @author Rein Raudjärv
 * 
 * @see ServletIntegration
 * 
 * TODO remove RebelServletConextFacade if possible
 */
public class CatalinaPlugin implements Plugin {
  public void preinit() {
    ClassLoader cl = getClass().getClassLoader();

    IntegrationFactory.getInstance().addIntegrationProcessor(
        cl,
        "org.apache.catalina.core.ApplicationContext", 
        new ServletContextImplClassBytecodeProcessor());
    IntegrationFactory.getInstance().addIntegrationProcessor(
        cl,
        "org.apache.catalina.loader.WebappLoader", 
        new WebappLoaderCBP());
    IntegrationFactory.getInstance().addIntegrationProcessor(
        cl,
        "org.apache.catalina.core.ApplicationContextFacade", 
        new ApplicationContextFacadeCBP());
  }

  public boolean checkDependencies(ClassLoader cl, ClassResourceSource crs) {
    return crs.getClassResource("org.apache.catalina.core.ApplicationContext") != null;
  }
 
  public String getId() {
    return "catalina_plugin";
  }
  
  public String getName() {
    return null;
  }
  
  public String getDescription() {
    return null;
  }

  public String getAuthor() {
    return null;
  }

  public String getWebsite() {
    return null;
  }  
}
